import sys

sys.path.append("../service")

import time

import yaml
import threading

import requests as requests
from flask import Flask, request
from flask_cors import CORS
from pymongo import MongoClient
from decouple import config

from dataclasses import dataclass

from app.view import Robokassa
from model.scheme import Merchant, Order

@dataclass
class Invoice:
    email: str
    invoice_id: str
    amount: int
    signature: str

active_invoices = []

app = Flask(__name__)
CORS(app, resources={r"/mg/pderm/*": {"origins": "*"}})

real_auth_token = "dBNSFaf9hwHEx7ZLRS9mFaUM7MvD4U2ZVfdHZJ8qgKELpQ8MJL"

last_invoice_id = 0

client = MongoClient('mongodb://root:uikfbHRyq9gwEVQPLxjV8Dm6Gpu3fr@mongo:27017/')
db = client["niko_event_reg"]
collection_requests = db["requests"]
collection_system = db["system"]

merchant = Merchant(config("MERCHANT_ID"), [config("MERCHANT_PASSWORD_TEST_1"), config("MERCHANT_PASSWORD_TEST_2")])

robokassa_payment = Robokassa(merchant)

# def check_invoices():
#     global active_invoices
#     while True:
#         for invoice in active_invoices:
#             if invoice["time"] < 0:
#                 active_invoices.remove(invoice)
#                 print(f"Removed invoice {invoice['order_id']}")
#                 continue
#             invoice["time"] -= 1
#         time.sleep(5)

def update_invoice_id():
    global last_invoice_id
    max_value = 2147000000
    while True:
        if last_invoice_id >= max_value:
            last_invoice_id = 0
        collection_system.update_one({}, {"$set": {"last_invoice_id": last_invoice_id}})
        time.sleep(60)


def send_email(from_email:str, to_email:str, subject:str, message = None, message_file=None):
    api_key = config("DASHA_API_KEY")

    if not api_key:
        raise Exception("No token provided")

    if not message and not message_file:
        raise Exception("No message provided")

    if message_file:
        with open(message_file, "r") as f:
            message = f.read()
        if not message:
            raise Exception("Message file is empty")

    url = "https://api.dashamail.com/?method=transactional.send"

    body = {
    "api_key": api_key,
    "to": to_email,
    "message": message,
    "from_email": from_email,
    "subject": subject
    }

    response = requests.post(url, json=body)
    response = response.json()

    print(response)

    if response["response"]["msg"]["err_code"] != 0:
        raise Exception("Error sending email")

    return response

@app.route('/mg/pderm/crm/robokassa/result', methods=['POST'])
def process_result():
    global collection_requests

    proxy_url = "https://pediatric-dermatology.ru/payment/robokassa/"

    data = request.json

    invoice_id = data["invoice_id"]
    amount = data["OutSum"]
    signature = data["SignatureValue"]

    if find_result := collection_requests.find_one({"invoice_id": invoice_id, "out_sum": amount}):
        #checksum_result = robokassa_payment.check_success_payment_post(data)
        checksum_result = True
        if not checksum_result:
            print(f"Processed {invoice_id}, failure")
            return "bad_sign", 200
        collection_requests.update_one({"invoice_id": invoice_id, "out_sum": amount}, {"$set": {"status": "finished"}})
        #collection_requests.delete_one({"invoice_id": invoice_id, "out_sum": amount})
        print(f"Processed {invoice_id}, succeed")
        return f"OK{invoice_id}", 200

    response = requests.post(proxy_url, json=data)

    return response.content, response.status_code

@app.route('/mg/pderm/crm/getLink', methods=['POST'])
def getLink():
    global last_invoice_id
    global active_invoices

    headers = request.headers
    auth_header = headers.get("Authorization", None)

    if not auth_header:
        return "No auth header", 404

    auth_token = auth_header.split()[1]

    if auth_token != real_auth_token:
        return "Wrong token", 404

    body = request.get_json()

    email_to = body['email']
    customer = body['customer']
    event = body['event']

    transactions = []
    # initial_transaction = {
    #     "amountRub": body["transactions"]["initial"]["amountRub"],
    #     "status": body["transactions"]["initial"]["status"]
    # }
    initial_transaction = body["transactions"]["initial"]

    body["invoice_id"] = last_invoice_id
    body["status"] = "pending"
    body["out_sum"] = initial_transaction["amountRub"]

    collection_requests.insert_one(body)

    order = Order(str(last_invoice_id), event, initial_transaction["amountRub"])
    robokassa_url = robokassa_payment.generate_payment_link(order)

    last_invoice_id += 1



    with open("config.yaml", "r") as stream:
        try:
            yaml_data = yaml.safe_load(stream)["data"]
        except yaml.YAMLError as exc:
            print(exc)

    email_from = yaml_data["after_reg_email_from"]
    email_subject = yaml_data["after_reg_email_subject"]
    email_text_file = yaml_data["after_reg_email"]

    if not email_subject:
        raise Exception("No email subject provided")

    if not email_text_file:
        raise Exception("No email text provided")

    send_email(email_from, email_to, email_subject, message_file=email_text_file)

    # validate robokassa payment

    return robokassa_url, 200

if __name__ == '__main__':
#    mongo = MongoClient('mongodb://mongo:27017/')

#    db = mongo["niko_event_reg"]
#    collection_system = db["system"]

    if collection_system.count_documents({}) == 0:
        collection_system.insert_one({"last_invoice_id": 0})
    else:
        last_invoice_id = collection_system.find_one({})["last_invoice_id"]

    #thread_1 = threading.Thread(target=check_invoices)
    thread_2 = threading.Thread(target=update_invoice_id)

    #thread_1.start()
    thread_2.start()

    app.run(host="0.0.0.0")


