import sys
from urllib.parse import urlparse

sys.path.append("../service")

import time

import yaml
import threading

import requests as requests

import re

from flask import Flask, request
from flask_cors import CORS
from pymongo import MongoClient
from decouple import config

from dataclasses import dataclass

from app.view import Robokassa
from model.scheme import Merchant, Order


@dataclass
class Invoice:
    email: str
    invoice_id: str
    amount: int
    signature: str


with open("config.yaml", "r") as stream:
    try:
        yaml_data = yaml.safe_load(stream)["data"]
    except yaml.YAMLError as exc:
        print(exc)

active_invoices = []

app = Flask(__name__)
CORS(app, resources={r"/mg/pderm/*": {"origins": "*"}})

real_auth_token = "dBNSFaf9hwHEx7ZLRS9mFaUM7MvD4U2ZVfdHZJ8qgKELpQ8MJL"

last_invoice_id = 0

# client = MongoClient('mongodb://mongo:27017/')
client = MongoClient('mongodb://root:uikfbHRyq9gwEVQPLxjV8Dm6Gpu3fr@mongo:27017/')

db = client["niko_event_reg"]
collection_requests = db["requests"]
collection_system = db["system"]

#merchant = Merchant(config("MERCHANT_ID"), [config("MERCHANT_PASSWORD_1"), config("MERCHANT_PASSWORD_2")])
#merchant = Merchant(config("MERCHANT_ID"),[config("MERCHANT_PASSWORD_1"), config("MERCHANT_PASSWORD_2")] ,[config("MERCHANT_PASSWORD_TEST_1"), config("MERCHANT_PASSWORD_TEST_2")])

merchant = Merchant(login=config("MERCHANT_ID"), password=[config("MERCHANT_PASSWORD_1"), config("MERCHANT_PASSWORD_2")], test_password = [config("MERCHANT_PASSWORD_TEST_1"), config("MERCHANT_PASSWORD_TEST_2")])


robokassa_payment = Robokassa(merchant)


# def check_invoices():
#     global active_invoices
#     while True:
#         for invoice in active_invoices:
#             if invoice["time"] < 0:
#                 active_invoices.remove(invoice)
#                 print(f"Removed invoice {invoice['order_id']}")
#                 continue
#             invoice["time"] -= 1
#         time.sleep(5)

def parse_response(request: str) -> dict:
    """
    :param request: Link.
    :return: Dictionary.
    """
    params = {}

    for item in urlparse(request).query.split('&'):
        key, value = item.split('=')
        params[key] = value
    return params

def update_invoice_id():
    global last_invoice_id
    max_value = 2147000000
    while True:
        if last_invoice_id >= max_value:
            last_invoice_id = 0
        collection_system.update_one({}, {"$set": {"last_invoice_id": last_invoice_id}})
        time.sleep(60)


def send_email(from_email: str, to_email: str, subject: str, message=None, message_file=None, placeholders=None):
    api_key = config("DASHA_API_KEY")

    if not api_key:
        return False, "No token provided"

    if not message and not message_file:
        return False, "No message provided"

    if message_file:
        with open(message_file, "r") as f:
            message = f.read()
        if not message:
            return False, "Message file is empty"

    if placeholders:
        result, message = preprocess_email_text(message, placeholders)
        if not result:
            return False, message

    url = "https://api.dashamail.com/?method=transactional.send"

    body = {
        "api_key": api_key,
        "to": to_email,
        "message": message,
        "from_email": from_email,
        "subject": subject
    }

    response = requests.post(url, json=body)
    response = response.json()

    if response["response"]["msg"]["err_code"] != 0:
        return False, response

    return True, response


def preprocess_email_text(message_text: str, placeholders: dict):
    for key in placeholders:
        message_text = message_text.replace(f"{{{key}}}", placeholders[key])

    placeholders_in_text = re.findall(r"{(.*?)}", message_text)
    if placeholders_in_text:
        return False, "Not all placeholders were replaced"

    return True, message_text


@app.route('/mg/pderm/crm/robokassa/result', methods=['POST'])
def process_result():
    global collection_requests

    proxy_url = "https://pediatric-dermatology.ru/payment/robokassa/"

    # get body
    data = request.form

    # data = {value[0]:value[1] for value in data}
    #
    # print(data, flush=True)

    invoice_id = int(data["inv_id"])
    amount = int(float(data["OutSum"]))

    find_result = collection_requests.find_one({"invoice_id": invoice_id, "out_sum": amount})

    #if find_result := collection_requests.find_one({"invoice_id": invoice_id, "out_sum": amount}):
    after_pay_email_subject = find_result.get("after_pay_email_subject", "Спасибо за оплату")
    after_pay_email_body = find_result.get("after_pay_email_body", None)
    after_pay_email_placeholders = find_result.get("after_pay_email_placeholders", None)
    after_pay_email_from = yaml_data["after_pay_email_from"]
    email_to = find_result.get("user_email", None)

    print(f"""after_pay_email_subject: '{after_pay_email_subject}'
    after_pay_email_body: '{after_pay_email_body}',
    after_pay_email_placeholders: '{after_pay_email_placeholders}',
    after_pay_email_from: '{after_pay_email_from}',
    email_to: '{email_to}'""", flush=True)


    if after_pay_email_body:
        email_result, email_response = send_email(after_pay_email_from,
                                                  email_to,
                                                  after_pay_email_subject,
                                                  message=after_pay_email_body,
                                                  placeholders=after_pay_email_placeholders)
    else:
        email_result, email_response = send_email(after_pay_email_from,
                                                  email_to,
                                                  after_pay_email_subject,
                                                  message_file=yaml_data["after_pay_email"],
                                                  placeholders=after_pay_email_placeholders)

    print(email_result, email_response, flush=True)

    #checksum_result = robokassa_payment.check_success_payment_post(data)
    checksum_result = True
    if not checksum_result:
        print(f"Processed {invoice_id}, failure")
        return "bad_sign", 200
    collection_requests.update_one({"invoice_id": invoice_id, "out_sum": amount}, {"$set": {"status": "finished"}})
    # collection_requests.delete_one({"invoice_id": invoice_id, "out_sum": amount})
    print(f"Processed {invoice_id}, succeed")
    return f"OK{invoice_id}", 200

    # response = requests.post(proxy_url, json=data)
    #
    # return response.content, response.status_code


@app.route('/mg/pderm/crm/getLink', methods=['POST'])
def getLink():
    global last_invoice_id
    global active_invoices

    headers = request.headers
    auth_header = headers.get("Authorization", None)

    if not auth_header:
        return "No auth header", 404

    auth_token = auth_header.split()[1]

    if auth_token != real_auth_token:
        return "Wrong token", 404

    body = request.get_json()

    email_to = body['email']
    customer = body['customer']
    event = body['event']

    after_reg_email_subject = body.get('after_reg_email_subject', yaml_data["after_reg_email_subject"])
    after_reg_email_body = body.get('after_reg_email_body', None)
    after_reg_email_placeholders = body.get('after_reg_email_placeholders', None)
    after_pay_email_subject = body.get('after_pay_email_subject', yaml_data["after_pay_email_subject"])
    after_pay_email_body = body.get('after_pay_email_body', None)
    after_pay_email_placeholders = body.get('after_pay_email_placeholders', None)
    after_pay_email_from = body.get('after_pay_email_from', yaml_data["after_pay_email_from"])

    initial_transaction = body["transactions"]["initial"]

    body["invoice_id"] = last_invoice_id
    body["status"] = "pending"
    body["out_sum"] = initial_transaction["amountRub"]
    body["after_pay_email_subject"] = after_pay_email_subject
    body["after_pay_email_body"] = after_pay_email_body
    body["after_pay_email_placeholders"] = after_pay_email_placeholders
    body["user_email"] = email_to

    is_test = body.get('is_test', None)

    collection_requests.insert_one(body)

    order = Order(str(last_invoice_id), event, initial_transaction["amountRub"])
    robokassa_url = robokassa_payment.generate_payment_link(order, test=is_test)

    last_invoice_id += 1

    after_reg_email_from = yaml_data["after_reg_email_from"]
    after_reg_email_text_file = yaml_data["after_reg_email"]

    if not after_reg_email_subject:
        return "No subject provided for first email", 404

    if not (after_reg_email_text_file or after_reg_email_body):
        return "No text provided for first email", 404

    if not after_reg_email_placeholders:
        after_reg_email_placeholders = {"payment_url": robokassa_url,
                                        "event": event}
    else:
        after_reg_email_placeholders["payment_url"] = robokassa_url
        after_reg_email_placeholders["event"] = event

    if after_reg_email_body:
        email_result, email_response = send_email(after_reg_email_from,
                                                  email_to,
                                                  after_reg_email_subject,
                                                  message=after_reg_email_body,
                                                  placeholders=after_reg_email_placeholders)
    else:
        email_result, email_response = send_email(after_reg_email_from,
                                                  email_to,
                                                  after_reg_email_subject,
                                                  message_file=after_reg_email_text_file,
                                                  placeholders=after_reg_email_placeholders)

    if not email_result:
        return email_response, 404

    return robokassa_url, 200


if __name__ == '__main__':
    if collection_system.count_documents({}) == 0:
        collection_system.insert_one({"last_invoice_id": 0})
    else:
        last_invoice_id = collection_system.find_one({})["last_invoice_id"]

    # thread_1 = threading.Thread(target=check_invoices)
    thread_2 = threading.Thread(target=update_invoice_id)

    # thread_1.start()
    thread_2.start()

    app.run(host="0.0.0.0")
