import os
import soundfile as sf

from flask import Flask, flash, request, redirect, url_for, jsonify
from werkzeug.utils import secure_filename
from allosaurus.app import read_recognizer
from uuid import uuid4

UPLOAD_FOLDER = '.'
ALLOWED_EXTENSIONS = {'wav', 'mp3', "ogg"}

app = Flask(__name__)
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER


def allowed_file(filename):
    return '.' in filename and \
           filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

@app.route("/popi", methods=["POST"])
def parse_by_url():
    data = request.json

    print(123)

    if not "url" in data.keys():
        return jsonify({"result": "error", "info": "No url detected"}), 404

    file = requests.get(data["url"], stream=True)
    filename_extention = data["url"].split(".")[-1]

    generated_filename = str(uuid4()) + "." + filename_extention

    with open(generated_filename, "wb") as handle:
        for data in file.iter_content():
            handle.write(data)

    model = read_recognizer()

    result = model.recognize(generated_filename, lang_id="rus")

    os.remove(generated_filename)

    return jsonify({"result": "success", "info": result}), 200

@app.route('/', methods=['GET', 'POST'])
def upload_file():
    if request.method == 'POST':
        # check if the post request has the file part
        if 'file' not in request.files:
            return jsonify({"result": "error", "info": "No file detected"}), 404
        file = request.files['file']
        # If the user does not select a file, the browser submits an
        # empty file without a filename.
        if file.filename == '':
            return jsonify({"result": "error", "info": "No selected file"}), 404
        if file and allowed_file(file.filename):
            filename = secure_filename(file.filename)
            filename_extention = filename.split(".")[-1]

            generated_filename = str(uuid4()) + "." + filename_extention

            file.save(os.path.join(app.config['UPLOAD_FOLDER'], generated_filename))

            if filename_extention != "wav":
                old_filename = generated_filename
                data, samplerate = sf.read(generated_filename)

                generated_filename = str(uuid4()) + "." + "wav"

                sf.write(generated_filename, data, samplerate)
                os.remove(old_filename)

            model = read_recognizer()

            result = model.recognize(generated_filename, lang_id="rus", timestamp=True)

            os.remove(generated_filename)

            return jsonify({"result": "success", "info": result}), 200




    return '''
    <!doctype html>
    <title>Upload new File</title>
    <h1>Upload new File</h1>
    <form method=post enctype=multipart/form-data>
      <input type=file name=file>
      <input type=submit value=Upload>
    </form>
    '''

if __name__ == "__main__":
    app.run()
