import flask
from flask import request, jsonify
import os

from string import digits, ascii_letters
import random

from werkzeug.utils import secure_filename

app = flask.Flask(__name__)

letters = digits + ascii_letters

# if not os.path.exists("./scripts"):
#     os.mkdir("./scripts")

if not os.path.exists("./temp"):
    os.mkdir("./temp")

@app.route('/tg/functions/add', methods=['POST'])
def add():
    """
    Method for adding a new function to the server, the function is sent as a file with "file" key
    :return: 200 if the function was added successfully, 400 if no file was sent, 500 if an error occurred
    """
    try:

        if not (user_token := check_token(request.headers.get('Authorization'))):
            return "Unauthorized", 401

        if not os.path.exists(f"./{user_token}"):
            os.mkdir(f"./{user_token}")

        file = flask.request.files['file']

        print(file)


        if not file:
            return "No file", 400

        filename = secure_filename(file.filename)

        file_token = gen_token()

        #insert split file name on name and extension
        filename = os.path.splitext(filename)[0] + "_" + file_token + os.path.splitext(filename)[1]

        with open(f"./{user_token}/" + filename, 'wb') as f:
            f.write(file.read())

        return "OK", 200
    except Exception as e:
        return f"Error", 500

@app.route('/tg/app', methods=['GET'])
def web_app():
    html = """
    <form action="/tg/functions/add" method="post" enctype="multipart/form-data">
        <input type="file" name="file" />
        <input type="submit" value="Submit" />
    </form>
    """
    return html

@app.route("/tg/functions/remove/<name>", methods=['GET'])
def remove(name):
    """
    Method for removing a function from the server
    :param name: name of the function to remove without extension
    :return: 200 if the function was removed successfully, 404 if the function was not found, 500 if an error occurred
    """
    try:
        
        if not (user_token := check_token(request.headers.get('Authorization'))):
            return "Unauthorized", 401
        
        if not os.path.exists(f"./{user_token}"):
            return "Folder not found", 404
        
        avalable_files = os.listdir(f"./{user_token}")

        file = [f for f in avalable_files if f.startswith(name)]
        if not file:
            return "File not found", 404
        
        filename = file[0]

        os.remove(f"./{user_token}/" + filename)

        return "OK", 200
    except Exception as e:
        return f"Error: {e}", 500

@app.route('/tg/functions/list', methods=['GET'])
def list():
    """
    Method for listing all the functions on the server
    :return: 200 and list of avalable functions if the function was removed successfully, 500 if an error occurred
    """

    if not (user_token := check_token(request.headers.get('Authorization'))):
        return "Unauthorized", 401

    print(user_token)

    try:
        if not os.path.exists(f"./{user_token}"):
            return "Directory not found", 404
        files = os.listdir(f"./{user_token}")
        return flask.jsonify([dict(zip(["name", "token"], os.path.splitext(f)[0].split("_")))  for f in files]), 200
    except Exception as e:
        return f"Error: {e}", 500

@app.route('/tg/functions/run/<name>', methods=['GET'])
def run(name):
    """
    Method for running a function on the server
    :param name: name of the function to run without extension
    :return: 200 if the function was run successfully, 404 if the function was not found, 500 if an error occurred
    """
    try:
        if not (user_token := check_token(request.headers.get('Authorization'))):

            if len(name) == 64:
                #find unique file in all subdirectories that contains the token in name
                for root, dirs, files in os.walk("."):
                    for file in files:
                        if name in file:
                            file = os.path.join(root, file)
                            return run_file(file)
                return "File not found", 404
            else:
                return "Unauthorized", 401

        if not os.path.exists(f"./{user_token}"):
            return "Not Found", 404

        avalable_files = os.listdir(f"./{user_token}")

        file = [f for f in avalable_files if f.split("_")[0] == name]
        if not file:
            return "File not found", 404

        file = f"./{user_token}/" + file[0]
        return run_file(file)
    except Exception as e:
        print(e)
        return f"Error: {e}", 500

@app.route('/tg/functions/python/install_requirements', methods=['POST'])
def install_requirements():
    """
    Method for installing requirements for a python function, the requirements.txt file is sent as a file with "file" key
    :return: 200 if the requirements were installed successfully, 500 if an error occurred
    """
    try:
        file = request.files['file']
        with open("./temp/" + file.filename, 'wb') as f:
            f.write(file.read())

        os.system("pip install -r ./scripts/" + file.filename)
        os.remove("./temp/" + file.filename)

        return "OK", 200
    except Exception as e:
        return f"Error: {e}", 500

def run_file(file):
    ext = os.path.splitext(file)[1]

    match ext:
        case ".py":
            run_python(file)
        case ".cpp":
            run_cpp(file)
        case _:
            return "File not supported", 400
    return "OK", 200

def run_python(file):
    try:
        os.system("python " + file)
        return "OK", 200

    except Exception as e:
        return f"Error: {e}", 500

def run_cpp(file):
    try:
        os.system("g++ ./scripts/" + file + " -o ./temp/" + file + ".out")
        os.system("./scripts/" + file + ".out")
        return "OK", 200
    except Exception as e:
        return f"Error: {e}", 500

def gen_token(length=64):
    return ''.join(random.choice(letters) for i in range(length))

def check_token(token):

    if not token:
        return None

    if "bearer" in token.lower():
        token = token.split(" ")[1]

    with open("./tokens.txt", "r") as f:
        tokens = f.readlines()

    return token if token in tokens else None

if __name__ == '__main__':
    app.run(host='0.0.0.0')
